/*
 * Synchronisation support.
 *
 * Copyright (C) 2018, 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/c/util/cap_alloc.h>

#include <pthread.h>
#include <pthread-l4.h>

#include <ipc/server.h>
#include "sync_support.h"



/* Obtain a capability for access to the current thread. */

long SyncSupport::ensure_client()
{
  if (l4_is_invalid_cap(_client_ep))
    return ipc_server_new_gate_for_thread(&_client_ep,
                                          pthread_l4_cap(pthread_self()),
                                          l4_umword_t(0));
  return L4_EOK;
}

/* Synchronisation methods. */

long SyncSupport::begin()
{
  long err = ensure_client();
  if (err)
    return err;

  queue()->join(_client_ep);
  return L4_EOK;
}

long SyncSupport::begin_first()
{
  long err = ensure_client();
  if (err)
    return err;

  queue()->join_first(_client_ep);
  return L4_EOK;
}

long SyncSupport::end()
{
  if (l4_is_valid_cap(_client_ep))
  {
    queue()->leave();
    l4re_util_cap_free_um(_client_ep);
    _client_ep = L4_INVALID_CAP;
  }

  return L4_EOK;
}

long SyncSupport::yield()
{
  if (l4_is_valid_cap(_client_ep))
    queue()->yield();

  return L4_EOK;
}
